# **************************************************************************** #
#                           This file is part of:                              #
#                                   METTLE                                     #
#                           https://bitsmiths.co.za                            #
# **************************************************************************** #
#  Copyright (C) 2015 - 2022 Bitsmiths (Pty) Ltd.  All rights reserved.        #
#   * https://bitbucket.org/bitsmiths_za/mettle.git                            #
#                                                                              #
#  Permission is hereby granted, free of charge, to any person obtaining a     #
#  copy of this software and associated documentation files (the "Software"),  #
#  to deal in the Software without restriction, including without limitation   #
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,    #
#  and/or sell copies of the Software, and to permit persons to whom the       #
#  Software is furnished to do so, subject to the following conditions:        #
#                                                                              #
#  The above copyright notice and this permission notice shall be included in  #
#  all copies or substantial portions of the Software.                         #
#                                                                              #
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  #
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    #
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL     #
#  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER  #
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     #
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         #
#  DEALINGS IN THE SOFTWARE.                                                   #
# **************************************************************************** #

from .database import Database

from .gen_sql       import GenSql
from .gen_sql_mssql import GenSqlMSSql


class DBMSSql(Database):

    def __init__(self):
        Database.__init__(self)


    def name(self) -> str:
        return 'mssql'


    def _db_generator(self) -> GenSql:
        return GenSqlMSSql()


    def _sequence_pre_insert(self) -> bool:
        return False


    def _sequence_post_insert(self) -> bool:
        return True


    def stored_proc_support(self) -> bool:
        return True


    def _table_name(self, gen_info: dict) -> str:
        if gen_info['db.schema'] == '':
            return '[%s]' % gen_info['table'].name

        return '[%s].[%s]' % (gen_info['db.schema'], gen_info['table'].name)


    def _column_name(self, cname: str) -> str:
        return '[%s]' % cname


    def _sql_lock_one(self, gen_info: dict) -> str:
        sql  = self._sql_select_one(gen_info)
        ssql = sql.lower()
        idx  = 0

        while True:
            idx = ssql.find("where", idx)

            if idx <= 0 or idx + 5 >= len(ssql):
                break

            if ssql[idx - 1].isspace() and ssql[idx + 6].isspace():
                return '%s WITH (UPDLOCK, NOWAIT, INDEX(PK_%s)) %s' % (sql[0:idx], gen_info['table'].name, sql[idx:])

            idx += 5

        raise Exception("Cannot lock a row wihtout a where clause! [table:{0}, sql:{1}]", self._table_name(gen_info), sql)
